#include <WiFi.h>
#include <DNSServer.h>
#include <WebServer.h>
#include <WiFiManager.h> 
#include <ModbusIP_ESP8266.h>

ModbusIP mb; // create modbus device
WiFiServer server(80);

byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED};  
byte ip[] = {192, 168, 0, 50}; 

#define DEBUG

// Current time
unsigned long currentTime = millis();
// Previous time
unsigned long previousTime = 0; 
// Define timeout time in milliseconds (example: 2000ms = 2s)
const long timeoutTime = 2000;
char bfr[100];

void setup() {
  Serial.begin(115200);

  WiFiManager wifiManager;
  wifiManager.autoConnect("MPG_Handwheel");

#ifdef DEBUG
  Serial.begin(115200);
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
#endif

  server.begin();
  
  mb.server();
  
  // create modbus holding registers
  mb.addHreg(55);
  mb.addHreg(56);
}

void loop() {
  mb.task(); // modbus polling function

  WiFiClient client = server.available();   // Listen for incoming clients

  if (client) {                             // If a new client connects,
    Serial.println("New Client.");          // print a message out in the serial port
    
    // Display the HTML web page
    client.println("<!DOCTYPE html><html>");
    client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
    client.println("<link rel=\"icon\" href=\"data:,\">");
    // CSS to style the on/off buttons 
    // Feel free to change the background-color and font-size attributes to fit your preferences
    client.println("<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}");
    client.println(".button { background-color: #4CAF50; border: none; color: white; padding: 16px 40px;");
    client.println("text-decoration: none; font-size: 30px; margin: 2px; cursor: pointer;}");
    client.println(".button2 {background-color: #555555;}</style></head>");
    
    // Web Page Heading
    client.println("<body><h1>ESP32 Web Server</h1>");

    getDROValue(55, 56);
    
    // Display X DRO 
    client.println("<p>X: ");
    client.println(bfr);
    client.println("<p>");
             
    client.println("</body></html>");
    
    // The HTTP response ends with another blank line
    client.println(); 
  }
}

float getDROValue(byte hreg1, byte hreg2) {
  int16_t numPrefix = mb.Hreg(hreg1); //read first holding register
  int16_t numPostfix = mb.Hreg(hreg2); // read second holding register
  float droVal = numPrefix + (numPostfix / 10000.0); // convert registers into decimal

  snprintf(bfr, sizeof(bfr), "%f", droVal);
  //dtostrf(droVal, 8, 4, bfr);
    
  Serial.println(bfr);
}
